@@ -4,7 +4,6 @@ Contains agents related to crystal-structure discovery
 and hypothesizing
 """
 
-import time
 import abc
 import json
 import os
@@ -12,25 +11,20 @@ from copy import deepcopy
 from multiprocessing import cpu_count
 from collections import OrderedDict
 
-import pandas as pd
-import tensorflow as tf
-import gpflow
-from gpflow.ci_utils import ci_niter
 import numpy as np
 from qmpy.analysis.thermodynamics.phase import Phase, PhaseData
 from camd.analysis import PhaseSpaceAL, ELEMENTS
 from camd.agent.base import HypothesisAgent, QBC
-from camd.utils.data import filter_dataframe_by_composition
+from camd.utils.data import filter_dataframe_by_composition, get_default_featurizer
 from pymatgen.core.composition import Composition
 
 from sklearn.gaussian_process.kernels import RBF, ConstantKernel
 from sklearn.gaussian_process import GaussianProcessRegressor
 from sklearn.linear_model import LinearRegression
-from sklearn.model_selection import cross_val_score, KFold, train_test_split
+from sklearn.model_selection import cross_val_score, KFold
 from sklearn.preprocessing import StandardScaler
 from sklearn.pipeline import Pipeline
 from sklearn.ensemble import AdaBoostRegressor, BaggingRegressor
-from sklearn.cluster import MiniBatchKMeans
 from sklearn.metrics import pairwise_distances
 
 # TODO: Adaptive N_query and subsampling of candidate space
@@ -51,6 +45,7 @@ class StabilityAgent(HypothesisAgent, metaclass=abc.ABCMeta):
         seed_data=None,
         n_query=1,
         hull_distance=0.0,
+        feature_labels=None,
         parallel=cpu_count(),
     ):
         """
@@ -60,6 +55,7 @@ class StabilityAgent(HypothesisAgent, metaclass=abc.ABCMeta):
             n_query (int): number of hypotheses to generate
             hull_distance (float): hull distance as a criteria for
                 which to deem a given material as "stable"
+            feature_labels ([str]): list of columns to filter for features in ML
             parallel (bool, int): whether to use multiprocessing
                 for phase stability analysis, if an int, sets the n_jobs
                 parameter as well.  If a bool, sets n_jobs to cpu_count()
@@ -72,6 +68,10 @@ class StabilityAgent(HypothesisAgent, metaclass=abc.ABCMeta):
         self.hull_distance = hull_distance
         self.pd = None
         self.parallel = parallel
+        if feature_labels is not None:
+            self.feature_labels = feature_labels
+        else:
+            self.feature_labels = get_default_featurizer().feature_labels()
 
         # These might be able to go into the base class
         self.cv_score = np.nan
@@ -90,9 +90,8 @@ class StabilityAgent(HypothesisAgent, metaclass=abc.ABCMeta):
         self.pd = PhaseData()
         # Filter seed data by relevant chemsys
         if chemsys:
-            total_comp = Composition(chemsys.replace('-', ''))
-            filtered = filter_dataframe_by_composition(
-                self.seed_data, total_comp)
+            total_comp = Composition(chemsys.replace("-", ""))
+            filtered = filter_dataframe_by_composition(self.seed_data, total_comp)
         else:
             filtered = self.seed_data
 
@@ -129,24 +128,16 @@ class StabilityAgent(HypothesisAgent, metaclass=abc.ABCMeta):
         # Note: In the drop command, we're ignoring errors for
         #   brevity.  We should watch this, because we may not
         #   drop everything we intend to.
-        drop_columns = [
-            "Composition",
-            "N_species",
-            "delta_e",
-            "pred_delta_e",
-            "pred_stability",
-            "stability",
-            "is_stable",
-            "structure",
-        ]
         if candidate_data is not None:
             self.candidate_data = candidate_data
-            X_cand = candidate_data.drop(drop_columns, axis=1, errors="ignore")
+            X_cand = self.get_features(self.candidate_data)
         else:
             X_cand = None
         if seed_data is not None:
-            self.seed_data = seed_data
-            X_seed = self.seed_data.drop(drop_columns, axis=1, errors="ignore")
+            # Filter seed data with No delta_e, indicates a failure
+            # TODO: probably worth considering putting this somewhere else
+            self.seed_data = seed_data.dropna(subset=["delta_e"])
+            X_seed = self.get_features(self.seed_data)
             y_seed = self.seed_data["delta_e"]
         else:
             X_seed, y_seed = None, None
@@ -189,7 +180,7 @@ class StabilityAgent(HypothesisAgent, metaclass=abc.ABCMeta):
         ]
 
         # Refresh and copy seed PD filtered by candidates
-        all_comp = self.candidate_data['Composition'].sum()
+        all_comp = self.candidate_data["Composition"].sum()
         pd_ml = deepcopy(self.get_pd(all_comp))
         pd_ml.add_phases(candidate_phases)
         space_ml = PhaseSpaceAL(bounds=ELEMENTS, data=pd_ml)
@@ -205,11 +196,21 @@ class StabilityAgent(HypothesisAgent, metaclass=abc.ABCMeta):
 
         return self.candidate_data
 
+    def get_features(self, dataframe):
+        """
+        Gets features of a dataframe
+
+        Returns:
+            (pd.DataFrame): dataframe filtered to include only features
+        """
+        return dataframe[self.feature_labels]
+
 
 class QBCStabilityAgent(StabilityAgent):
     """
     Agent which uses QBC to determine optimal hypotheses
     """
+
     def __init__(
         self,
         candidate_data=None,
@@ -221,6 +222,7 @@ class QBCStabilityAgent(StabilityAgent):
         training_fraction=0.5,
         model=None,
         n_members=10,
+        feature_labels=None,
     ):
         """
         Args:
@@ -245,13 +247,16 @@ class QBCStabilityAgent(StabilityAgent):
             n_query=n_query,
             hull_distance=hull_distance,
             parallel=parallel,
+            feature_labels=feature_labels,
         )
 
         self.alpha = alpha
         self.model = model
         self.n_members = n_members
         self.qbc = QBC(
-            n_members=n_members, training_fraction=training_fraction, model=model,
+            n_members=n_members,
+            training_fraction=training_fraction,
+            model=model,
         )
 
     def get_hypotheses(self, candidate_data, seed_data=None, retrain_committee=True):
@@ -306,6 +311,7 @@ class AgentStabilityML5(StabilityAgent):
         parallel=cpu_count(),
         model=None,
         exploit_fraction=0.5,
+        feature_labels=None,
     ):
         """
         Args:
@@ -326,6 +332,7 @@ class AgentStabilityML5(StabilityAgent):
             n_query=n_query,
             hull_distance=hull_distance,
             parallel=parallel,
+            feature_labels=feature_labels,
         )
 
         self.model = model or LinearRegression()
@@ -352,12 +359,12 @@ class AgentStabilityML5(StabilityAgent):
         cv_score = cross_val_score(
             pipeline,
             X_seed,
-            self.seed_data["delta_e"],
+            y_seed,
             cv=KFold(5, shuffle=True),
             scoring="neg_mean_absolute_error",
         )
         self.cv_score = np.mean(cv_score) * -1
-        pipeline.fit(X_seed, self.seed_data["delta_e"])
+        pipeline.fit(X_seed, y_seed)
 
         expected = pipeline.predict(X_cand)
 
@@ -396,6 +403,7 @@ class GaussianProcessStabilityAgent(StabilityAgent):
         hull_distance=0.0,
         parallel=cpu_count(),
         alpha=0.5,
+        feature_labels=None,
     ):
         """
         Args:
@@ -415,6 +423,7 @@ class GaussianProcessStabilityAgent(StabilityAgent):
             n_query=n_query,
             hull_distance=hull_distance,
             parallel=parallel,
+            feature_labels=feature_labels,
         )
         self.multiprocessing = parallel
         self.alpha = alpha
@@ -471,207 +480,6 @@ class GaussianProcessStabilityAgent(StabilityAgent):
         return within_hull.head(self.n_query)
 
 
-class SVGProcessStabilityAgent(StabilityAgent):
-    """
-    Stochastic variational gaussian process stability agent for Big Data.
-
-    The computational complexity of this algorithm scales as O(M^3)
-    compared to O(N^3) of standard GP, where N is the number of data points
-    and M is the number of inducing points (M<<N).
-
-    The default parameters are optimized to deliver a compromise between
-    compute-time and model accuracy for data sets with up to 25 to 40
-    thousand examples (e.g. the ICSD seed data). For bigger systems,
-    parameter M may need to be reduced. For smaller systems, it can be
-    increased, if higher accuracy is desired.  Inducing point locations
-    are determined using k-means clustering.
-
-    References:
-        Hensman, James, Nicolo Fusi, and Neil D. Lawrence. "Gaussian
-            processes for big data." Uncertainty in Artificial
-            Intelligence (2013).
-        Kingma, Diederik P., and Jimmy Ba. "Adam: A method for stochastic
-            optimization." arXiv preprint arXiv:1412.6980 (2014).
-
-    """
-
-    def __init__(
-        self,
-        candidate_data=None,
-        seed_data=None,
-        n_query=1,
-        hull_distance=0.0,
-        parallel=cpu_count(),
-        alpha=0.5,
-        M=600,
-        maxiter=20000
-    ):
-        """
-        Args:
-            candidate_data (DataFrame): data about the candidates
-            seed_data (DataFrame): data which to fit the Agent to
-            n_query (int): number of hypotheses to generate
-            hull_distance (float): hull distance as a criteria for
-                which to deem a given material as "stable"
-            parallel (bool): whether to use multiprocessing
-                for phase stability analysis
-            alpha (float): weighting factor for the stdev in making
-                best-case predictions of the stability
-            M (int): number of inducing points associated with the
-                SVGProcess
-            maxiter (int): number of maximum iterations of the SVG
-                process
-        """
-        super(SVGProcessStabilityAgent, self).__init__(
-            candidate_data=candidate_data,
-            seed_data=seed_data,
-            n_query=n_query,
-            hull_distance=hull_distance,
-            parallel=parallel,
-        )
-        self.alpha = alpha
-        self.M = M
-        self.maxiter = maxiter
-
-        # Define non-argument SVG-specific attributes
-        self.kernel = gpflow.kernels.RBF(273) * gpflow.kernels.Constant(273)
-        self.mean_f = gpflow.mean_functions.Constant()
-        self.logger = None
-        self.model = None
-        self.pred_y = None
-        self.pred_std = None
-
-    def get_hypotheses(self, candidate_data, seed_data=None):
-        """
-        Get hypotheses method for SVGProcessStabilityAgent.
-
-        Code used from gpflow examples for big data, see:
-
-        https://github.com/GPflow/GPflow/blob/develop/doc/source/notebooks/advanced/gps_for_big_data.pct.py
-
-        Args:
-            candidate_data (pandas.DataFrame): dataframe of candidates
-            seed_data (pandas.DataFrame): dataframe of prior data on
-                which to fit model
-
-        Returns:
-            (pandas.DataFrame): top candidates from the algorithm
-
-        """
-        X_cand, X_seed, y_seed = self.update_data(candidate_data, seed_data)
-
-        # Test model performance first.  Note we avoid doing CV to
-        # reduce compute time. We simply do a 1-way split 80:20 (train:test)
-        X_train, X_test, y_train, y_test = train_test_split(
-            X_seed, y_seed, test_size=0.2, random_state=42
-        )
-        scaler = StandardScaler()
-        X_train_scaled = scaler.fit_transform(X_train)
-
-        # Do a k-means clustering and use cluster centers as inducing points.
-        cls = MiniBatchKMeans(n_clusters=self.M, batch_size=200)
-        cls.fit(X_train_scaled)
-        Z = cls.cluster_centers_
-        _y = np.array(y_train.to_list())
-        mu = np.mean(_y)
-        sig = np.std(_y)
-        print(Z, _y.shape)
-        print(sig, mu)
-        model = gpflow.models.SVGP(
-            self.kernel,
-            gpflow.likelihoods.Gaussian(),
-            Z,
-            mean_function=self.mean_f,
-        )
-        elbo = tf.function(model.elbo)
-
-        # TensorFlow re-traces & compiles a `tf.function`-wrapped method at *every* call
-        # if the arguments are numpy arrays instead of tf.Tensors. Hence:
-        tensor_data = tuple(map(tf.convert_to_tensor, (X_seed, pd.DataFrame(y_seed))))
-        elbo(tensor_data)  # run it once to trace & compile
-        minibatch_size = 100
-
-        # We turn off training for inducing point locations
-        gpflow.utilities.set_trainable(model.inducing_variable, False)
-
-        train_dataset = tf.data.Dataset.from_tensor_slices((X_train_scaled, pd.DataFrame(y_train))) \
-            .repeat() \
-            .shuffle(len(X_train_scaled))
-
-        def run_adam(model, iterations):
-            """
-            Utility function running the Adam optimizer
-
-            :param model: GPflow model
-            :param interations: number of iterations
-            """
-            # Create an Adam Optimizer action
-            logf = []
-            train_iter = iter(train_dataset.batch(minibatch_size))
-            training_loss = model.training_loss_closure(train_iter, compile=True)
-            optimizer = tf.optimizers.Adam()
-
-            @tf.function
-            def optimization_step():
-                optimizer.minimize(training_loss, model.trainable_variables)
-
-            for step in range(iterations):
-                optimization_step()
-                if step % 10 == 0:
-                    elbo = -training_loss().numpy()
-                    logf.append(elbo)
-            return logf
-
-        print("training")
-        t0 = time.time()
-        run_adam(model, ci_niter(self.maxiter))
-        print("elapsed time: ", time.time() - t0)
-
-        pred_y, pred_v = model.predict_y(scaler.transform(X_test))
-        pred_y = pred_y * sig + mu
-        self.cv_score = np.mean(np.abs(pred_y - y_test.to_numpy().reshape(-1, 1)))
-        print("cv score", self.cv_score)
-        self.model = model
-
-        # Overall model
-        scaler = StandardScaler()
-        X_scaled = scaler.fit_transform(X_seed)
-        cls = MiniBatchKMeans(n_clusters=self.M, batch_size=200)
-        cls.fit(X_scaled)
-        Z = cls.cluster_centers_
-        _y = np.array(y_seed.to_list())
-        mu = np.mean(_y)
-        sig = np.std(_y)
-        model = gpflow.models.SVGP(
-            self.kernel,
-            gpflow.likelihoods.Gaussian(),
-            Z,
-            mean_function=self.mean_f,
-        )
-        maxiter = gpflow.ci_utils.ci_niter(self.maxiter)
-        run_adam(model, maxiter)
-        print(self.model)
-        self.model = model
-
-        # GP makes predictions for Hf and uncertainty*alpha on candidate data
-        pred_y, pred_v = model.predict_y(scaler.transform(X_cand))
-        pred_y = pred_y * sig + mu
-        self.pred_y = np.array(pred_y).reshape(-1,)
-        self.pred_std = (np.array(pred_v) ** 0.5).reshape(-1,)
-
-        expected = self.pred_y - self.pred_std * self.alpha
-        print("expected improv", expected)
-
-        # Update candidate data dataframe with predictions
-        self.update_candidate_stabilities(expected, sort=True, floor=-6.0)
-
-        # Find the most stable ones up to n_query within hull_distance
-        stability_filter = self.candidate_data["pred_stability"] <= self.hull_distance
-        within_hull = self.candidate_data[stability_filter]
-
-        return within_hull.head(self.n_query)
-
-
 class BaggedGaussianProcessStabilityAgent(StabilityAgent):
     """
     An ensemble GP learner that can handle relatively large
@@ -698,6 +506,7 @@ class BaggedGaussianProcessStabilityAgent(StabilityAgent):
         n_estimators=8,
         max_samples=5000,
         bootstrap=False,
+        feature_labels=None,
     ):
         """
         Args:
@@ -717,6 +526,7 @@ class BaggedGaussianProcessStabilityAgent(StabilityAgent):
             n_query=n_query,
             hull_distance=hull_distance,
             parallel=parallel,
+            feature_labels=feature_labels,
         )
 
         self.alpha = alpha
@@ -823,6 +633,7 @@ class AgentStabilityAdaBoost(StabilityAgent):
         exploit_fraction=0.5,
         diversify=False,
         dynamic_alpha=False,
+        feature_labels=None,
     ):
         """
         Args:
@@ -856,6 +667,7 @@ class AgentStabilityAdaBoost(StabilityAgent):
             n_query=n_query,
             hull_distance=hull_distance,
             parallel=parallel,
+            feature_labels=feature_labels,
         )
         self.model = model
         self.exploit_fraction = exploit_fraction
@@ -880,15 +692,14 @@ class AgentStabilityAdaBoost(StabilityAgent):
         """
         X_cand, X_seed, y_seed = self.update_data(candidate_data, seed_data)
 
-        steps = [("scaler", StandardScaler()), ("ML", self.model)]
+        adaboost = AdaBoostRegressor(
+            base_estimator=self.model, n_estimators=self.n_estimators
+        )
+        steps = [("scaler", StandardScaler()), ("ML", adaboost)]
         pipeline = Pipeline(steps)
 
-        adaboost = AdaBoostRegressor(
-            base_estimator=pipeline, n_estimators=self.n_estimators
-        )
-
         cv_score = cross_val_score(
-            adaboost,
+            pipeline,
             X_seed,
             y_seed,
             cv=KFold(3, shuffle=True),
@@ -904,7 +715,7 @@ class AgentStabilityAdaBoost(StabilityAgent):
         overall_adaboost = AdaBoostRegressor(
             base_estimator=self.model, n_estimators=self.n_estimators
         )
-        overall_adaboost.fit(X_scaled, self.seed_data["delta_e"])
+        overall_adaboost.fit(X_scaled, y_seed)
 
         X_cand = scaler.transform(X_cand)
         expected = overall_adaboost.predict(X_cand)
@@ -931,7 +742,10 @@ class AgentStabilityAdaBoost(StabilityAgent):
         n_exploitation = int(self.n_query * self.exploit_fraction)
         if self.diversify:
             to_compute = diverse_quant(
-                within_hull.index.tolist(), n_exploitation, self.candidate_data
+                within_hull.index.tolist(),
+                n_exploitation,
+                self.candidate_data,
+                feature_filter=self.feature_labels,
             )
         else:
             to_compute = within_hull.head(n_exploitation).index.tolist()
@@ -964,7 +778,7 @@ class AgentStabilityAdaBoost(StabilityAgent):
         return np.array(stds)
 
 
-def diverse_quant(points, target_length, df, quantiles=None):
+def diverse_quant(points, target_length, df, quantiles=None, feature_filter=None):
     """
     Diversify a sublist by eliminating entries based on comparisons
     with quantiles threshold and Euclidean distance.
@@ -1014,20 +828,16 @@ def diverse_quant(points, target_length, df, quantiles=None):
         _df = df.sample(6000)
     else:
         _df = df
-    drop_columns = [
-        "Composition",
-        "N_species",
-        "delta_e",
-        "pred_delta_e",
-        "pred_stability",
-        "structure"
-    ]
+    if feature_filter is not None:
+        _df = _df[feature_filter]
     scaler = StandardScaler()
-    X = scaler.fit_transform(_df.drop(drop_columns, axis=1, errors="ignore"))
+    X = scaler.fit_transform(_df)
     flatD = pairwise_distances(X).flatten()
 
     _df2 = df.loc[points]
-    X = scaler.transform(_df2.drop(drop_columns, axis=1, errors="ignore"))
+    if feature_filter is not None:
+        _df2 = _df2[feature_filter]
+    X = scaler.transform(_df2)
     D = pairwise_distances(X)
 
     remove_len = len(points) - target_length
